/*(c) Copyright 2015 Pivotal Software, Inc. All Rights Reserved.*/
'use strict';

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _setImmediate2 = require('babel-runtime/core-js/set-immediate');

var _setImmediate3 = _interopRequireDefault(_setImmediate2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var classnames = require('classnames');
var React = require('react');

var types = React.PropTypes;

var DOWN_KEY = 40;
var ENTER_KEY = 13;
var ESC_KEY = 27;
var TAB_KEY = 9;
var UP_KEY = 38;

function waitForRelatedTarget() {
  return new _promise2.default(function (resolve) {
    (0, _setImmediate3.default)(function () {
      resolve(document.activeElement);
    });
  });
}

function withRelatedTarget(callback) {
  return function () {
    var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(e) {
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (e.relatedTarget) {
                _context.next = 5;
                break;
              }

              e = (0, _extends3.default)({}, e);
              _context.next = 4;
              return waitForRelatedTarget.call(this);

            case 4:
              e.relatedTarget = _context.sent;

            case 5:
              return _context.abrupt('return', callback.call(this, e));

            case 6:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  }();
}

var AutocompleteInput = React.createClass({
  displayName: 'AutocompleteInput',

  propTypes: {
    $autocomplete: types.object,
    autoFocus: types.bool,
    children: function children(props, name) {
      if (props[name] && props[name].length) return new Error('AutocompleteInput can only wrap one element');
    },

    disabled: types.bool,
    hideList: types.func,
    onClick: types.func,
    onFocus: types.func,
    onPick: types.func,
    onPicking: types.func,
    onSearch: types.func,
    scrollIntoView: types.func
  },

  statics: {
    DOWN_KEY: DOWN_KEY,
    ENTER_KEY: ENTER_KEY,
    ESC_KEY: ESC_KEY,
    TAB_KEY: TAB_KEY,
    UP_KEY: UP_KEY
  },

  getDefaultProps: function getDefaultProps() {
    return { autoFocus: null };
  },


  blur: withRelatedTarget(function (_ref2) {
    var relatedTarget = _ref2.relatedTarget;

    if (relatedTarget && relatedTarget.classList && relatedTarget.classList.contains('autocomplete-item')) return;
    this.props.hideList();
  }),

  change: function change(e) {
    var _this = this;

    var value = e.currentTarget.value;

    this.props.onSearch(value, function (suggestedValues) {
      _this.props.$autocomplete.merge({ hidden: false, highlightedSuggestion: 0, value: value, suggestedValues: suggestedValues }).flush();
    });
  },
  keyDown: function keyDown(e) {
    var _this2 = this,
        _keyCodes;

    var keyCode = e.keyCode;

    var _props$$autocomplete$ = this.props.$autocomplete.get();

    var highlightedSuggestion = _props$$autocomplete$.highlightedSuggestion;
    var suggestedValues = _props$$autocomplete$.suggestedValues;
    var _props$onPicking = this.props.onPicking;
    var onPicking = _props$onPicking === undefined ? function () {
      return suggestedValues;
    } : _props$onPicking;


    var pickItem = function pickItem() {
      var selectableSuggestions = onPicking(suggestedValues);
      e && keyCode === ENTER_KEY && e.preventDefault();
      _this2.props.$autocomplete.merge({ highlightedSuggestion: -1, hidden: true }).flush();
      _this2.props.onPick(selectableSuggestions[highlightedSuggestion] || { value: _this2.props.$autocomplete.get('value') });
    };

    var keyCodes = (_keyCodes = {}, (0, _defineProperty3.default)(_keyCodes, DOWN_KEY, function () {
      var selectableSuggestions = onPicking(suggestedValues);
      _this2.props.$autocomplete.merge({ hidden: false, highlightedSuggestion: Math.min(highlightedSuggestion + 1, selectableSuggestions.length - 1) });
      _this2.props.scrollIntoView();
    }), (0, _defineProperty3.default)(_keyCodes, UP_KEY, function () {
      _this2.props.$autocomplete.merge({ highlightedSuggestion: Math.max(highlightedSuggestion - 1, -1) });
      _this2.props.scrollIntoView();
    }), (0, _defineProperty3.default)(_keyCodes, TAB_KEY, pickItem), (0, _defineProperty3.default)(_keyCodes, ENTER_KEY, pickItem), (0, _defineProperty3.default)(_keyCodes, ESC_KEY, function () {
      _this2.props.$autocomplete.merge({ highlightedSuggestion: -1, hidden: true });
    }), (0, _defineProperty3.default)(_keyCodes, 'noop', function noop() {}), _keyCodes);

    keyCodes[keyCode in keyCodes ? keyCode : 'noop']();
  },
  renderDefault: function renderDefault(props) {
    return React.createElement('input', (0, _extends3.default)({}, props, { className: classnames('autocomplete-input', 'form-control', props.className), type: 'search', value: props.value, 'aria-label': props.placeholder }));
  },
  render: function render() {
    var _props = this.props;
    var autoFocus = _props.autoFocus;
    var children = _props.children;
    var $autocomplete = _props.$autocomplete;
    var props = (0, _objectWithoutProperties3.default)(_props, ['autoFocus', 'children', '$autocomplete']);

    if (!$autocomplete) return null;

    var _$autocomplete$get = $autocomplete.get();

    var value = _$autocomplete$get.value;

    var otherProps = { autoFocus: autoFocus, value: value, onBlur: this.blur, onChange: this.change, onKeyDown: this.keyDown };
    props = (0, _extends3.default)({}, props, otherProps);
    if (!children) return this.renderDefault(props);
    children = React.Children.map(children, function (e) {
      return React.cloneElement(e, props);
    });
    return React.createElement(
      'div',
      null,
      children
    );
  }
});

module.exports = AutocompleteInput;